"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
from wsgidav import util
from wsgidav.middleware import BaseMiddleware


class AbstractFilter(BaseMiddleware):
    """
    A wireframe implementation of BaseMiddleware. Consider overriding handle_request() method to add an actual logic
    """

    def __init__(self, application, config):
        self._application = application
        self._config = config

    def __call__(self, environ, start_response):
        # Intercept start_response        
        sub_app_start_response = util.SubAppStartResponse()

        self.handle_request(environ)
                
        response_started = False
        app_iter = self._application(environ, sub_app_start_response)
        for v in app_iter:
            # Start response (the first time)
            if not response_started:
                # Success!
                start_response(sub_app_start_response.status,
                               sub_app_start_response.response_headers,
                               sub_app_start_response.exc_info)
            response_started = True

            yield v
                
        # Close out iterator
        if hasattr(app_iter, "close"):
            app_iter.close()

        # Start response (if it hasn't been done yet)
        if not response_started:
            # Success!
            start_response(sub_app_start_response.status,
                           sub_app_start_response.response_headers,
                           sub_app_start_response.exc_info)

        return

    def handle_request(self, environ):
        raise NotImplemented()