"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
import urlparse

from wsgidav import util
from wsgidav.dav_error import DAVError

from fileman.middleware.base_filter import AbstractFilter

FILTER_BY_PARAM = 'filter_by'
FILTER_VALUE = 'filter_value'
FILTERING_KEY = 'filtering.value'
QUERY_STRING = 'QUERY_STRING'


_logger = util.getModuleLogger(__name__)


class FilteringFilter(AbstractFilter):
    def handle_request(self, environ):
        if QUERY_STRING not in environ:
            return

        raw_value = environ[QUERY_STRING]
        q = urlparse.parse_qs(raw_value)
        filter_by = self.extract(q, FILTER_BY_PARAM)

        if not filter_by:
            # Do nothing if filtering is not given
            return

        filter_value = self.extract(q, FILTER_VALUE)

        self._assert(filter_by in ['exact_name', 'contains_name'])
        self._assert(filter_value is not None)

        environ[FILTERING_KEY] = {FILTER_BY_PARAM: filter_by,
                                  FILTER_VALUE: filter_value}

    def _assert(self, predicate):
        if not predicate:
            raise DAVError(400)

    def extract(self, q, param):
        if param not in q:
            return None
        value = q[param]
        self._assert(isinstance(value, list) and len(value) == 1)
        return value[0]