"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
import urlparse

from wsgidav import util
from wsgidav.dav_error import DAVError

from fileman.middleware.base_filter import AbstractFilter

AMOUNT_PARAM = 'amount'
OFFSET_PARAM = 'offset'
PAGING_KEY = 'paging_filter.value'
QUERY_STRING = 'QUERY_STRING'

_logger = util.getModuleLogger(__name__)


class PagingFilter(AbstractFilter):
    def handle_request(self, environ):
        if QUERY_STRING in environ:
            raw_value = environ[QUERY_STRING]
            q = urlparse.parse_qs(raw_value)
            if (OFFSET_PARAM in q) and (AMOUNT_PARAM in q):

                amt = q[AMOUNT_PARAM]
                offset = q[OFFSET_PARAM]
                self._assert(isinstance(amt, list) and len(amt) == 1)
                self._assert(isinstance(offset, list) and len(offset) == 1)

                amt = int(amt[0])
                offset = int(offset[0])

                environ[PAGING_KEY] = {AMOUNT_PARAM: amt,
                                       OFFSET_PARAM: offset}

    def _assert(self, predicate):
        if not predicate:
            raise DAVError(400)