"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
from wsgidav import util
from wsgidav.dav_error import DAVError

from fileman.middleware.base_filter import AbstractFilter
from fileman.middleware.roots.parser import AccessibleRootsParser


ACCESSIBLE_ROOTS_KEY = 'accessible_roots.value'

ACCESSIBLE_ROOTS = 'HTTP_X_AC_ACCESSIBLE_ROOTS'

_logger = util.getModuleLogger(__name__)


class AccessibleRootsFilter(AbstractFilter):
    def handle_request(self, environ):
        if ACCESSIBLE_ROOTS in environ:
            raw_value = environ[ACCESSIBLE_ROOTS]
            _logger.debug("%s header found. Value: '%s'" % (ACCESSIBLE_ROOTS, raw_value))
            roots = self.parse_accessible_roots(raw_value)
            environ[ACCESSIBLE_ROOTS_KEY] = roots
        else:
            self.handle_no_accessible_roots()

    def parse_accessible_roots(self, raw_value):
        parser = AccessibleRootsParser()
        value = parser.parse(raw_value)
        return value

    def handle_no_accessible_roots(self):
        raise DAVError(400, 'Header X-AC-Accessible-Roots is required')
