"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
from fileman.middleware.roots.parser import PathSplitter


class Matcher(object):
    """
    Matches the given path against the Accessible Roots setting
    """

    def __init__(self):
        super(Matcher, self).__init__()
        self.splitter = None

    def match_roots(self, path, roots):
        assert isinstance(path, basestring)
        assert isinstance(roots, list)

        path_parsed = self._parse_path(path)
        for root in roots:
            root_parsed = self._parse_path(root)
            pref = self._get_common_prefix(root_parsed, path_parsed)
            if len(pref) == len(root_parsed):
                # The path is somewhere under the given root. Approve.
                return True
            elif len(pref) == len(path_parsed):
                # The given path is fully contained within the root. E.g. root = /home/adaptive, path = /home
                return True
        return False

    def _get_common_prefix(self, root_parsed, path_parsed):

        common = []
        for i in xrange(min(len(root_parsed), len(path_parsed))):
            if root_parsed[i] == path_parsed[i]:
                common.append(root_parsed[i])
            else:
                break
        return common

    def _get_splitter(self):
        if not self.splitter:
            self.splitter = PathSplitter()
        return self.splitter

    def _parse_path(self, path):
        splitter = self._get_splitter()
        result = splitter.parse(path)
        return result
