"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""


class BaseSplitter(object):
    def parse(self, value):

        assert isinstance(value, basestring)
        i = 0
        result = []
        while i < len(value):
            next_dir, i = self.read_next_token(value, i)
            i = self.consume_separator(value, i)
            if next_dir:
                result.append(next_dir)
        return result

    def read_next_token(self, value, i):
        """
        Starting from index i, the function scans right until the end of the token.
        :param value:
        :param i: Current index to start from
        :return: Tuple: the path read and the new index value where the algorithm has stopped.
        """
        done = False
        result = ''
        while i < len(value) and not done:
            ch = value[i]
            if ch in self.get_separators():
                done = True
            else:
                if ch == '\\':
                    i += 1
                    ch = value[i]
                result += ch
                i += 1

        return result, i

    def consume_separator(self, value, i):
        """
        Starting from index i, the function switches it right unless it points to a non-separator character/
        :param value: String to scan through
        :param i: current index to start from.
        :return: New index value pointing either to the next non-separator or to the end of the line (i = len(value)).
        """
        while (i < len(value)) and (value[i] in self.get_separators()):
            i += 1

        return i

    def get_separators(self):
        raise NotImplemented()


class AccessibleRootsParser(BaseSplitter):
    def get_separators(self):
        return set(':')


class PathSplitter(BaseSplitter):
    def get_separators(self):
        return set('/')