"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
import urlparse

from wsgidav import util
from wsgidav.dav_error import DAVError

from fileman.middleware.base_filter import AbstractFilter

ORDER_BY_PARAM = 'sort_by'
ORDER_DIRECTION_PARAM = 'dir'
ORDERING_KEY = 'ordering.value'
QUERY_STRING = 'QUERY_STRING'


_logger = util.getModuleLogger(__name__)


class SortingFilter(AbstractFilter):
    def handle_request(self, environ):
        if QUERY_STRING not in environ:
            return

        raw_value = environ[QUERY_STRING]
        q = urlparse.parse_qs(raw_value)
        order_by = self.extract(q, ORDER_BY_PARAM)

        if not order_by:
            # Do nothing if ordering is not given
            return

        direction = self.extract(q, ORDER_DIRECTION_PARAM) or 'ASC'

        direction = direction.upper()

        self._assert(direction in ('ASC', 'DESC'))

        environ[ORDERING_KEY] = {ORDER_BY_PARAM: order_by,
                                 ORDER_DIRECTION_PARAM: direction}

    def _assert(self, predicate):
        if not predicate:
            raise DAVError(400)

    def extract(self, q, param):
        if param not in q:
            return None
        value = q[param]
        self._assert(isinstance(value, list) and len(value) == 1)
        return value[0]