"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
import os
from wsgidav import util
from wsgidav.fs_dav_provider import FilesystemProvider
from fileman.provider.impersonated import AccessibleRootsHelper, ImpersonatedFolderResource, ImpersonatedFileResource
from fileman.provider.impersonated.utils import is_home_request, get_home_dir

_logger = util.getModuleLogger(__name__)


class AcDAVProvider(FilesystemProvider):
    def getResourceInst(self, path, environ):
        _logger.debug("Resource is requested: %s" % path)
        self._count_getResourceInst += 1
        if isinstance(path, unicode):
            path = path.encode('utf-8')

        fp = self._locToFilePath(path)
        _logger.debug("Local path: %s" % fp)

        if not os.path.lexists(fp):
            _logger.debug("File doesn't exist: %s" % fp)
            if is_home_request(path):
                _logger.debug("Special case: home directory is requested")

                # [KN] Note that this method is invoked before our impersonation filter. That is why we should use
                # the explicit username to get the correct home
                username = environ["http_authenticator.username"]
                fp = get_home_dir(username)
                _logger.debug("Home is: %s" % fp)
            else:
                return None

        # [KN] Since getResourceInst is called in from request_server.py:doGET method, we
        # should check whether the requested resource is available through the accessible roots.
        ar_helper = AccessibleRootsHelper(environ)
        if not ar_helper.accessible_roots_satisfy(fp):
            # In this case 404 Not Found is returned to the client.
            _logger.debug("Requested path is restricted by Accessible Roots policy: %s" % fp)
            return None

        if os.path.isdir(fp):
            return ImpersonatedFolderResource(path, environ, fp)
        return ImpersonatedFileResource(path, environ, fp)

