"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
import errno

from wsgidav.dav_error import DAVError


def fs_permission_required(fn):
    def wrap(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except (IOError, OSError) as e:
            if e.errno in [errno.EPERM, errno.EACCES]:
                # Returning '403 Bad request' if the user has no file permission
                # (changed from 401 by results of IRIS-2542)
                raise DAVError(403)
            elif e.errno == errno.ENOENT:
                # No such file found -> HTTP 404
                raise DAVError(404)
            elif e.errno == errno.ENAMETOOLONG:
                # file name too long -> HTTP 414 Request-URI too long
                raise DAVError(414)
            else:
                raise e

    return wrap
