"""
Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
"""
import re


class FilterPredicate(object):
    """
    Base filter predicate
    """
    def matches(self, item):
        raise NotImplemented


class TruePredicate(FilterPredicate):
    def matches(self, item):
        return True


class ExactNamePredicate(FilterPredicate):
    def __init__(self, required_name):
        super(ExactNamePredicate, self).__init__()
        self.required_name = required_name

    def matches(self, item):
        return item == self.required_name


class ContainsNamePredicate(FilterPredicate):
    def __init__(self, required_name):
        super(ContainsNamePredicate, self).__init__()
        escaped_val = re.escape(required_name)
        self.regex = re.compile('^.*' + escaped_val + '.*$')

    def matches(self, item):
        r = self.regex
        return r.match(item) is not None


class Filter(object):
    def __init__(self, predicate):
        super(Filter, self).__init__()
        assert isinstance(predicate, FilterPredicate)
        self.predicate = predicate

    def filter_out(self, file_list):
        for item in file_list:
            if self.predicate.matches(item):
                yield item

