
#===============================================================================
#                          Variables and defaults
#===============================================================================

PROG="install-server-certs"
PERMISSIONS="0600"

install_dir=/opt/acfileman/etc
server_key_fname=server-key.pem
server_cert_fname=server-cert.pem
ca_cert_fname=ca-cert.pem
permissions="0600"
user=""

#===============================================================================
#                          Command line parsing
#===============================================================================

show_help()
{
        cat <<EOF
Usage: ./$PROG [options] <target dir>
Valid options are:
        -k, --serverkey      name of server key file in target directory
        -c, --servercert     name of server cert file in target directory
        -a, --cacert         name of CA cert file in target directory
	-p, --permissions    permissions for installed files
        -u, --user           specify user and group (in form user:group) for installed files
	-h, --help           show this text and exit
EOF
exit 1
}

TEMP=`getopt -o h,k:,c:,a:,p:,u: --long serverkey:,servercert:,cacert:,permissions:,user: -n $PROG -- "$@"`
eval set -- "$TEMP"

while :; do
    case "$1" in
	--) shift; break
	    ;;
        -k|--serverkey) shift ; server_key_fname="$1"
	    ;;
        -c|--servercert) shift ; server_cert_fname="$1"
	    ;;
        -a|--cacert) shift ; ca_cert_fname="$1"
	    ;;
        -p|--permissions) shift ; permissions="$1"
	    ;;
        -u|--user) shift ; user="$1"
	    ;;
        -h|--help) show_help
	    ;;
        *) echo "Unknown option" >&2 ; show_help
    esac
    shift
done

if [ "$#" -ne 1 ]; then
    show_help
fi

install_dir=$1

#===============================================================================
#                          Install certificates
#===============================================================================

mkdir -p "$install_dir" || { echo >&2 "Can't create directory for certificates: $install_dir. Aborting"; exit 1; }

create_file_with_perm()
{
    local content=$1
    local dest=$2
    local permissions=$3
    user=$4
    echo "$content" > $dest || { echo >&2 "Can't put certificate content to file: $dest. Aborting"; exit 1; }
    chmod $permissions $dest || { echo >&2 "Can't set permissions $permissions for file: $dest. Aborting"; exit 1; }
    if [ -n "$user" ]; then
	chown $user $dest || { echo >&2 "Can't change user and group to $user for file: $dest. Aborting"; exit 1; }
    fi
}

create_file_with_perm "$server_key" "$install_dir/$server_key_fname" $permissions "$user"
create_file_with_perm "$server_cert" "$install_dir/$server_cert_fname" $permissions "$user"
create_file_with_perm "$ca_cert" "$install_dir/$ca_cert_fname" $permissions "$user"
